@@ -1,5 +1,5 @@
 /*
- * This file produced from ./encodings.pl on Fri Nov 24 09:01:32 2023
+ * This file produced from ./encodings.pl on Thu May  9 16:52:34 2024
  */
 #ifdef TDS_ICONV_ENCODING_TABLES
 
@@ -137,6 +137,7 @@ static const CHARACTER_SET_ALIAS iconv_aliases[] = {
 	{                 "CP1361",   28 },
 	{                  "CP367",   92 },
 	{                  "CP437",   29 },
+	{                "CP65001",    1 },
 	{                  "CP819",    0 },
 	{                  "CP850",   30 },
 	{                  "CP862",   31 },
