@@ -35,9 +35,10 @@
 #include <string.h>
 #endif /* HAVE_STRING_H */
 
+#include <freetds/replacements.h>
+
 #include "cspublic.h"
 #include "ctlib.h"
-#include "replacements.h"
 
 #undef cs_dt_crack
 
@@ -314,27 +315,32 @@ Cleanup:
 }
 
 CS_RETCODE
-cs_ctx_alloc(CS_INT version, CS_CONTEXT ** ctx)
+cs_ctx_alloc(CS_INT version, CS_CONTEXT ** out_ctx)
 {
 	TDSCONTEXT *tds_ctx;
+	CS_CONTEXT *ctx;
 
-	tdsdump_log(TDS_DBG_FUNC, "cs_ctx_alloc(%d, %p)\n", version, ctx);
+	tdsdump_log(TDS_DBG_FUNC, "cs_ctx_alloc(%d, %p)\n", version, out_ctx);
 
-	*ctx = tds_new0(CS_CONTEXT, 1);
-	tds_ctx = tds_alloc_context(*ctx);
+	ctx = tds_new0(CS_CONTEXT, 1);
+	if (!ctx)
+		return CS_FAIL;
+	ctx->use_large_identifiers = _ct_is_large_identifiers_version(version);
+	tds_ctx = tds_alloc_context(ctx);
 	if (!tds_ctx) {
-		free(*ctx);
+		free(ctx);
 		return CS_FAIL;
 	}
-	(*ctx)->tds_ctx = tds_ctx;
-	if (tds_ctx->locale && !tds_ctx->locale->date_fmt) {
+	ctx->tds_ctx = tds_ctx;
+	if (tds_ctx->locale && !tds_ctx->locale->datetime_fmt) {
 		/* set default in case there's no locale file */
-		tds_ctx->locale->date_fmt = strdup(STD_DATETIME_FMT);
+		tds_ctx->locale->datetime_fmt = strdup(STD_DATETIME_FMT);
 	}
 
-	(*ctx)->login_timeout = -1;
-	(*ctx)->query_timeout = -1;
+	ctx->login_timeout = -1;
+	ctx->query_timeout = -1;
 
+	*out_ctx = ctx;
 	return CS_SUCCEED;
 }
 
@@ -466,14 +472,15 @@ cs_config(CS_CONTEXT * ctx, CS_INT action, CS_INT property, CS_VOID * buffer, CS
 }
 
 CS_RETCODE
-cs_convert(CS_CONTEXT * ctx, CS_DATAFMT * srcfmt, CS_VOID * srcdata, CS_DATAFMT * destfmt, CS_VOID * destdata, CS_INT * resultlen)
+_cs_convert(CS_CONTEXT * ctx, const CS_DATAFMT_COMMON * srcfmt, CS_VOID * srcdata,
+	    const CS_DATAFMT_COMMON * destfmt, CS_VOID * destdata, CS_INT * resultlen)
 {
 	TDS_SERVER_TYPE src_type, desttype;
-	int src_len, destlen, len, i = 0;
+	int src_len, destlen, len;
 	CONV_RESULT cres;
 	unsigned char *dest;
 	CS_RETCODE ret;
-	CS_INT dummy;
+	CS_INT dummy, datatype;
 	CS_VARCHAR *destvc = NULL;
 
 	tdsdump_log(TDS_DBG_FUNC, "cs_convert(%p, %p, %p, %p, %p, %p)\n", ctx, srcfmt, srcdata, destfmt, destdata, resultlen);
@@ -507,20 +514,23 @@ cs_convert(CS_CONTEXT * ctx, CS_DATAFMT * srcfmt, CS_VOID * srcdata, CS_DATAFMT
 
 	}
 
-	src_type = _ct_get_server_type(NULL, srcfmt->datatype);
+	datatype = srcfmt->datatype;
+	src_type = _ct_get_server_type(NULL, datatype);
 	if (src_type == TDS_INVALID_TYPE)
 		return CS_FAIL;
 	src_len = srcfmt->maxlength;
-	if (srcfmt->datatype == CS_VARCHAR_TYPE || srcfmt->datatype == CS_VARBINARY_TYPE) {
+	if (datatype == CS_VARCHAR_TYPE || datatype == CS_VARBINARY_TYPE) {
 		CS_VARCHAR *vc = (CS_VARCHAR *) srcdata;
 		src_len = vc->len;
 		srcdata = vc->str;
 	}
-	desttype = _ct_get_server_type(NULL, destfmt->datatype);
+
+	datatype = destfmt->datatype;
+	desttype = _ct_get_server_type(NULL, datatype);
 	if (desttype == TDS_INVALID_TYPE)
 		return CS_FAIL;
 	destlen = destfmt->maxlength;
-	if (destfmt->datatype == CS_VARCHAR_TYPE || destfmt->datatype == CS_VARBINARY_TYPE) {
+	if (datatype == CS_VARCHAR_TYPE || datatype == CS_VARBINARY_TYPE) {
 		destvc = (CS_VARCHAR *) destdata;
 		destlen  = sizeof(destvc->str);
 		destdata = destvc->str;
@@ -689,7 +699,7 @@ cs_convert(CS_CONTEXT * ctx, CS_DATAFMT * srcfmt, CS_VOID * srcdata, CS_DATAFMT
 	}
 
 	tdsdump_log(TDS_DBG_FUNC, "cs_convert() calling tds_convert\n");
-	len = tds_convert(ctx->tds_ctx, src_type, (TDS_CHAR*) srcdata, src_len, desttype, &cres);
+	len = tds_convert(ctx->tds_ctx, src_type, srcdata, src_len, desttype, &cres);
 
 	tdsdump_log(TDS_DBG_FUNC, "cs_convert() tds_convert returned %d\n", len);
 
@@ -737,8 +747,7 @@ cs_convert(CS_CONTEXT * ctx, CS_DATAFMT * srcfmt, CS_VOID * srcdata, CS_DATAFMT
 			destvc->len = len;
 			*resultlen = sizeof(*destvc);
 		}
-		for (i = len; i < destlen; i++)
-			dest[i] = '\0';
+		memset(dest + len, '\0', destlen - len);
 		break;
 	case SYBBIT:
 	case SYBBITN:
@@ -800,8 +809,7 @@ cs_convert(CS_CONTEXT * ctx, CS_DATAFMT * srcfmt, CS_VOID * srcdata, CS_DATAFMT
 			tdsdump_log(TDS_DBG_FUNC, "cs_convert() FMT_PADBLANK\n");
 			/* strcpy here can lead to a small buffer overflow */
 			memcpy(dest, cres.c, len);
-			for (i = len; i < destlen; i++)
-				dest[i] = ' ';
+			memset(dest + len, ' ', destlen - len);
 			*resultlen = destlen;
 			break;
 
@@ -809,8 +817,7 @@ cs_convert(CS_CONTEXT * ctx, CS_DATAFMT * srcfmt, CS_VOID * srcdata, CS_DATAFMT
 			tdsdump_log(TDS_DBG_FUNC, "cs_convert() FMT_PADNULL\n");
 			/* strcpy here can lead to a small buffer overflow */
 			memcpy(dest, cres.c, len);
-			for (i = len; i < destlen; i++)
-				dest[i] = '\0';
+			memset(dest + len, '\0', destlen - len);
 			*resultlen = destlen;
 			break;
 		case CS_FMT_UNUSED:
@@ -836,6 +843,13 @@ cs_convert(CS_CONTEXT * ctx, CS_DATAFMT * srcfmt, CS_VOID * srcdata, CS_DATAFMT
 	return (ret);
 }
 
+CS_RETCODE
+cs_convert(CS_CONTEXT * ctx, CS_DATAFMT * srcfmt, CS_VOID * srcdata, CS_DATAFMT * destfmt, CS_VOID * destdata, CS_INT * resultlen)
+{
+	return _cs_convert(ctx, _ct_datafmt_common(ctx, srcfmt), srcdata,
+			   _ct_datafmt_common(ctx, destfmt), destdata, resultlen);
+}
+
 CS_RETCODE
 cs_dt_crack_v2(CS_CONTEXT * ctx, CS_INT datetype, CS_VOID * dateval, CS_DATEREC * daterec)
 {
@@ -1438,10 +1452,10 @@ _cs_convert_not_client(CS_CONTEXT *ctx, const TDSCOLUMN *curcol, CONV_RESULT *co
 	}
 
 	if (convert_buffer) {
-		int len = tds_convert(ctx->tds_ctx, type, (TDS_CHAR *) *p_src,
+		int len = tds_convert(ctx->tds_ctx, type, *p_src,
 				      curcol->column_cur_size, desttype, convert_buffer);
 		if (len < 0)
-			return CS_ILLEGAL_TYPE; // TODO _csclient_msg ??
+			return CS_ILLEGAL_TYPE; /* TODO _csclient_msg ?? */
 		*p_src = (unsigned char *) convert_buffer;
 	}
 	return ct_type;
